/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2002 Frank Fesevur                                       //
//                                                                         //
//  This program is software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// OptionsWebServerPage.cpp : implementation file

#include "StdAfx.h"
#include "Dorgem.h"
#include "OptionsWebServerPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COptionsWebServerPage property page

IMPLEMENT_DYNCREATE(COptionsWebServerPage, CHelpPropertyPage)

COptionsWebServerPage::COptionsWebServerPage() : CHelpPropertyPage(COptionsWebServerPage::IDD)
{
  //{{AFX_DATA_INIT(COptionsWebServerPage)
  m_uPort = GetApp()->GetProfileInt(szGeneralWebServer, szPort, DEFAULT_WEB_PORT);
  m_uRefreshRate = GetApp()->GetProfileInt(szGeneralWebServer, szRefreshRate, 0);
  m_strLogFile = GetApp()->GetProfileString(szGeneralWebServer, szAccessLog);
  //}}AFX_DATA_INIT

  m_help.SetPopups("PopupsOptionsWebServer.txt");
}

COptionsWebServerPage::~COptionsWebServerPage()
{
}

void COptionsWebServerPage::DoDataExchange(CDataExchange* pDX)
{
  CHelpPropertyPage::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(COptionsWebServerPage)
  DDX_Control(pDX, IDC_SERVER_TYPE, m_cmbServerType);
  DDX_Text(pDX, IDC_PORT, m_uPort);
  DDX_Text(pDX, IDC_REFRESH, m_uRefreshRate);
  DDV_MinMaxUInt(pDX, m_uRefreshRate, 0, 60);
  DDX_Text(pDX, IDC_LOG_FILE, m_strLogFile);
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(COptionsWebServerPage, CHelpPropertyPage)
  //{{AFX_MSG_MAP(COptionsWebServerPage)
  ON_CBN_SELCHANGE(IDC_SERVER_TYPE, OnSelectionChangedServerType)
	ON_EN_CHANGE(IDC_PORT, SetModified)
	ON_EN_CHANGE(IDC_REFRESH, SetModified)
	ON_EN_CHANGE(IDC_LOG_FILE, SetModified)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
//

BOOL COptionsWebServerPage::OnInitDialog()
{
  CHelpPropertyPage::OnInitDialog();

  int iType = GetApp()->GetProfileInt(szGeneralWebServer, szWebServerType, WEB_SERVER_NONE);
  AddServerTypeString("None", WEB_SERVER_NONE, iType);
  AddServerTypeString("Still images", WEB_SERVER_STILL, iType);
  //AddServerTypeString("Streaming", WEB_SERVER_STREAM, iType);
  OnSelectionChangedServerType();

  // Reset the modified flag
  m_bModified = FALSE;

  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
//

void COptionsWebServerPage::OnOK()
{
  int iType, iIndex = m_cmbServerType.GetCurSel();
  if (iIndex == CB_ERR)
    iType = WEB_SERVER_NONE;
  else
    iType = m_cmbServerType.GetItemData(iIndex);

  GetApp()->WriteProfileInt(szGeneralWebServer, szWebServerType, iType);
  GetApp()->WriteProfileInt(szGeneralWebServer, szPort, m_uPort);
  GetApp()->WriteProfileInt(szGeneralWebServer, szRefreshRate, m_uRefreshRate);
  GetApp()->WriteProfileString(szGeneralWebServer, szAccessLog, m_strLogFile);

  if (iType == WEB_SERVER_NONE)
    GetApp()->StopWebServer();
  else
  {
    if (!GetApp()->IsWebServerRunning() || m_bModified)
      GetApp()->StartWebServer();
  }

  CHelpPropertyPage::OnOK();
}

/////////////////////////////////////////////////////////////////////////////
//

void COptionsWebServerPage::AddServerTypeString(LPCSTR szStr, int iValue, int iType)
{
  int iIndex = m_cmbServerType.AddString(szStr);
  if (iIndex == CB_ERR)
    return;

  m_cmbServerType.SetItemData(iIndex, iValue);
  if (iValue == iType)
    m_cmbServerType.SetCurSel(iIndex);
}

/////////////////////////////////////////////////////////////////////////////
//

void COptionsWebServerPage::OnSelectionChangedServerType()
{
  int iType, iIndex = m_cmbServerType.GetCurSel();
  if (iIndex == CB_ERR)
    iType = WEB_SERVER_NONE;
  else
    iType = m_cmbServerType.GetItemData(iIndex);

  BOOL bEnabled = TRUE;
  switch (iType)
  {
    case WEB_SERVER_NONE:
      bEnabled = FALSE;
      break;
    case WEB_SERVER_STILL:
    case WEB_SERVER_STREAM:
      bEnabled = TRUE;
      break;
  }
  GetDlgItem(IDC_PORT)->EnableWindow(bEnabled);
  GetDlgItem(IDC_REFRESH)->EnableWindow(bEnabled);
  GetDlgItem(IDC_LOG_FILE)->EnableWindow(bEnabled);

  SetModified();
}

/////////////////////////////////////////////////////////////////////////////
//

void COptionsWebServerPage::SetModified() 
{
  m_bModified = TRUE;
}
