/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2001 Frank Fesevur                                       //
//                                                                         //
//  This program is software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// StoreFile.cpp : implementation file

#include "StdAfx.h"
#include "StoreEvent.h"
#include "StoreFile.h"
#include "Dorgem.h"

#include "Tm.h"
#include "RegKey.h"
#include "RegistryEntry.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
//

CStoreFile::CStoreFile()
{
  m_iType = STORE_FILE;
}

/////////////////////////////////////////////////////////////////////////////
//

BOOL CStoreFile::Store()
{
  TRACE("[CStoreFile]Store(%s)\n", m_strName);

  if (!m_bEnable || m_bBusy)
    return(TRUE);
  m_bBusy = TRUE;

  CTM tmNow;
  CString strFile = tmNow.Format(m_strFile);

  BOOL bRet = CopyFile(m_strOrigFile, strFile, FALSE);
  if (!bRet)
  {
    m_strErrorMsg.Format("Failed to save '%s': %s", strFile,
                                   GetApp()->GetSystemError(GetLastError()));
    TRACE("Error: '%s'\n", m_strErrorMsg);
    Log(0, m_strErrorMsg);
  }

  if (bRet || m_bResetTime)
  {
    CTM tmNow;
    m_tLastUpdate = tmNow;
  }

  m_bBusy = FALSE;

  return(bRet);
}

/////////////////////////////////////////////////////////////////////////////
//

BOOL CStoreFile::ReadRegistry(CRegKey* pKey)
{
  ASSERT(pKey != NULL);

  Log(5, "Reading from registry");

  CStoreEvent::ReadRegistry(pKey);
  m_strFile = pKey->GetStringValue(szFile);

  Log(5, "Ready reading from registry");
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
//

BOOL CStoreFile::WriteRegistry(CRegKey* pKey)
{
  Log(5, "Writing to registry");

  CStoreEvent::WriteRegistry(pKey);

  // Store the information
  CRegKey reg;
  reg.Create(*pKey, m_strName);
  reg.SetStringValue(szFile, m_strFile);
  reg.Close();

  Log(5, "Ready writing to registry");
  return(TRUE);
}
