/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2002 Frank Fesevur                                       //
//                                                                         //
//  This program is software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// StoreProgram.cpp : implementation file

#include "StdAfx.h"
#include "Dorgem.h"
#include "StoreEvent.h"
#include "StoreProgram.h"

#include "Tm.h"
#include "RegKey.h"
#include "RegistryEntry.h"

#ifdef _DEBUG
//#define __NO_FTP__
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
//

CStoreProgram::CStoreProgram()
{
  m_iType = STORE_PROGRAM;
}

////////////////////////////////////////////////////////////////////////////
// Sort of generic StartProgram()

DWORD CStoreProgram::StartProgram(LPCSTR szCmdLine, BOOL bWaitFinish)
{
  TCHAR szCmd[_MAX_PATH];
  lstrcpy(szCmd, szCmdLine);

  TCHAR szPath[_MAX_PATH];
  GetCurrentDirectory(_MAX_PATH, szPath);

  STARTUPINFO si;
  ZeroMemory(&si, sizeof(STARTUPINFO));
  si.dwFlags = STARTF_USESHOWWINDOW;
  si.wShowWindow = SW_HIDE;
  si.cb = sizeof(STARTUPINFO);

  PROCESS_INFORMATION pi;
  ZeroMemory(&pi, sizeof(PROCESS_INFORMATION));

  DWORD dwReturn = NOERROR;
  if (CreateProcess(NULL, szCmd, NULL, NULL, FALSE, CREATE_DEFAULT_ERROR_MODE, NULL, szPath, &si, &pi))
  {
    if (bWaitFinish)
      WaitForSingleObject(pi.hProcess, INFINITE);
  }
  else
    dwReturn = GetLastError();

  return(dwReturn);
}

/////////////////////////////////////////////////////////////////////////////
//

BOOL CStoreProgram::Store()
{
  TRACE("[CStoreProgram]Store(%s)\n", m_strName);

  if (!m_bEnable || m_bBusy)
    return(TRUE);
  m_bBusy = TRUE;

  CString strTmp;
  strTmp.Format(m_strCommand, m_strOrigFile);

  BOOL bRet = (StartProgram(strTmp, TRUE) == 0);

  if (bRet || m_bResetTime)
  {
    CTM tmNow;
    m_tLastUpdate = tmNow;
  }
  m_bBusy = FALSE;

  return(bRet);
}

/////////////////////////////////////////////////////////////////////////////
//

BOOL CStoreProgram::ReadRegistry(CRegKey* pKey)
{
  ASSERT(pKey != NULL);

  Log(LOG_DEBUG, "Reading from registry");

  CStoreEvent::ReadRegistry(pKey);

  m_strCommand = pKey->GetStringValue(szCommand);

  Log(LOG_DEBUG, "Ready reading from registry");
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
//

BOOL CStoreProgram::WriteRegistry(CRegKey* pKey)
{
  Log(LOG_DEBUG, "Writing to registry");

  CStoreEvent::WriteRegistry(pKey);

  // Store the information
  CRegKey reg;
  reg.Create(*pKey, m_strName);
  reg.SetStringValue(szCommand, m_strCommand);
  reg.Close();

  Log(LOG_DEBUG, "Ready writing to registry");
  return(TRUE);
}
