/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Dorgem - Webcam Capture application                                    //
//  Copyright (C) 2002 Frank Fesevur                                       //
//                                                                         //
//  This program is software; you can redistribute it and/or modify   //
//  it under the terms of the GNU General Public License as published by   //
//  the Software Foundation; either version 2 of the License, or      //
//  (at your option) any later version.                                    //
//                                                                         //
//  This program is distributed in the hope that it will be useful,        //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of         //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           //
//  GNU General Public License for more details.                           //
//                                                                         //
//  You should have received a copy of the GNU General Public License      //
//  along with this program; if not, write to the Software            //
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// StoreProgramDlg.cpp : implementation file

#include "StdAfx.h"
#include "Dorgem.h"
#include "StoreProgramDlg.h"

#include "StoreEvent.h"
#include "StoreProgram.h"
#include "Tm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CStoreProgramDlg dialog

CStoreProgramDlg::CStoreProgramDlg(CWnd* pParent /*=NULL*/)
  : CHelpDialog(CStoreProgramDlg::IDD, pParent)
{
  //{{AFX_DATA_INIT(CStoreProgramDlg)
  m_strCommand = _T("");
  m_bEnable = TRUE;
  m_dwInterval = DEFAULT_INTERVAL;
  m_strName = _T("");
  m_bResetTime = TRUE;
  //}}AFX_DATA_INIT

  m_pEvent = NULL;

  m_help.SetPopups("PopupsStoreProgram.txt");
}

void CStoreProgramDlg::DoDataExchange(CDataExchange* pDX)
{
  CHelpDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CStoreProgramDlg)
  DDX_Control(pDX, IDC_INTERVAL_TYPE, m_cmbIntervalType);
  DDX_Text(pDX, IDC_COMMAND, m_strCommand);
  DDX_Check(pDX, IDC_ENABLE, m_bEnable);
  DDX_Text(pDX, IDC_INTERVAL, m_dwInterval);
  DDX_Text(pDX, IDC_NAME, m_strName);
  DDX_Check(pDX, IDC_RESET_TIME, m_bResetTime);
  //}}AFX_DATA_MAP

  DDV_MinChars(pDX, m_strName, 1);
  DDV_MinChars(pDX, m_strCommand, 1);

  if (m_pEvent == NULL)
    DDV_StoreEventUnique(pDX, m_strName);

  if (pDX->m_bSaveAndValidate)
  {
    if (m_strCommand.Find("%s") == -1)
    {
      MsgBoxEx("The command MUST contain %s, which\n"
               "is replaced with the name of the JPEG file!");
      pDX->Fail();
    }
  }
}

BEGIN_MESSAGE_MAP(CStoreProgramDlg, CHelpDialog)
  //{{AFX_MSG_MAP(CStoreProgramDlg)
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
//

BOOL CStoreProgramDlg::OnInitDialog()
{
  CHelpDialog::OnInitDialog();

  int iBetween = BETWEEN_SECS;
  if (m_pEvent != NULL)
  {
    time_t tInterval;
    m_pEvent->GetInterval(tInterval, iBetween);
    m_dwInterval = (DWORD) tInterval;

    GetDlgItem(IDC_NAME)->EnableWindow(FALSE);
    m_strName = m_pEvent->GetName();
    m_bEnable = m_pEvent->IsEnabled();
    m_strCommand = m_pEvent->GetCommand();
    m_bResetTime = m_pEvent->ResetTime();
    UpdateData(FALSE);
  }

  // Fill the interval type combo
  AddIntervalType("Hours", BETWEEN_HOURS, iBetween);
  AddIntervalType("Minutes", BETWEEN_MINS, iBetween);
  AddIntervalType("Seconds", BETWEEN_SECS, iBetween);

  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
//

void CStoreProgramDlg::OnOK()
{
  // Validate the entered values
  if (!UpdateData())
    return;

  // Is the interval type combo filled
  int iIndex = m_cmbIntervalType.GetCurSel();
  if (iIndex == LB_ERR)
    return;

  // Create an event
  if (m_pEvent == NULL)
  {
    m_pEvent = new CStoreProgram;
    ASSERT(m_pEvent != NULL);
  }

  // Store setting in the event
  m_pEvent->SetName(m_strName);
  m_pEvent->SetEnabled(m_bEnable);
  m_pEvent->SetInterval(m_dwInterval, m_cmbIntervalType.GetItemData(iIndex));
  m_pEvent->SetCommand(m_strCommand);
  m_pEvent->ResetTime(m_bResetTime);

  CHelpDialog::OnOK();
}

/////////////////////////////////////////////////////////////////////////////
//

void CStoreProgramDlg::AddIntervalType(LPCSTR szText, int iType, int iSel)
{
  int iIndex = m_cmbIntervalType.AddString(szText);
  m_cmbIntervalType.SetItemData(iIndex, iType);
  if (iSel == iType)
    m_cmbIntervalType.SetCurSel(iIndex);
}
